@@ -29,7 +29,6 @@
 #include <QTabWidget>
 #include <QLineEdit>
 #include <QFileDialog>
-#include <QMessageBox>
 #include <QSpinBox>
 #include <QListWidget>
 #include <QSettings>
@@ -40,6 +39,8 @@
 #include "widgets/IconLoader.h"
 #include "widgets/ColourMapComboBox.h"
 #include "widgets/ColourComboBox.h"
+#include "widgets/PluginPathConfigurator.h"
+#include "widgets/WidgetScale.h"
 #include "base/Preferences.h"
 #include "base/ResourceFinder.h"
 #include "layer/ColourMapper.h"
@@ -154,7 +155,7 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) :
             settings.value("overview-colour", m_overviewColour.name())
             .toString();
         m_overviewColour.setNamedColor(qcolorName);
-        cerr << "loaded colour " << m_overviewColour.name() << " from settings" << endl;
+        SVCERR << "loaded colour " << m_overviewColour.name() << " from settings" << endl;
     }
     settings.endGroup();
 
@@ -171,7 +172,7 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) :
     ColourComboBox *overviewColour = new ColourComboBox(false);
     int overviewColourIndex =
         ColourDatabase::getInstance()->getColourIndex(m_overviewColour);
-    cerr << "index = " << overviewColourIndex << " for colour " << m_overviewColour.name() << endl;
+    SVCERR << "index = " << overviewColourIndex << " for colour " << m_overviewColour.name() << endl;
     if (overviewColourIndex >= 0) {
         overviewColour->setCurrentIndex(overviewColourIndex);
     }
@@ -276,7 +277,7 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) :
     tempDirButton->setIcon(IconLoader().load("fileopen"));
     connect(tempDirButton, SIGNAL(clicked()),
             this, SLOT(tempDirButtonClicked()));
-    tempDirButton->setFixedSize(QSize(24, 24));
+    tempDirButton->setFixedSize(WidgetScale::scaleQSize(QSize(24, 24)));
 
     QCheckBox *showSplash = new QCheckBox;
     m_showSplash = prefs->getShowSplash();
@@ -328,7 +329,7 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) :
         QString f0 = f;
         f.replace("sonic-visualiser_", "").replace(".qm", "");
         if (f == f0) { // our expectations about filename format were not met
-            cerr << "INFO: Unexpected filename " << f << " in i18n resource directory" << endl;
+            SVCERR << "INFO: Unexpected filename " << f << " in i18n resource directory" << endl;
         } else {
             m_locales.push_back(f);
             QString displayText;
@@ -576,7 +577,17 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) :
     
     m_tabOrdering[AudioIOTab] = m_tabs->count();
     m_tabs->addTab(frame, tr("A&udio I/O"));
- 
+
+    // Plugins tab
+
+    m_pluginPathConfigurator = new PluginPathConfigurator(this);
+    m_pluginPathConfigurator->setPaths(PluginPathSetter::getPaths());
+    connect(m_pluginPathConfigurator, SIGNAL(pathsChanged()),
+            this, SLOT(pluginPathsChanged()));
+    
+    m_tabOrdering[PluginTab] = m_tabs->count();
+    m_tabs->addTab(m_pluginPathConfigurator, tr("&Plugins"));
+    
     // General tab
 
     frame = new QFrame;
@@ -903,6 +914,13 @@ PreferencesDialog::viewFontSizeChanged(int sz)
     m_applyButton->setEnabled(true);
 }
 
+void
+PreferencesDialog::pluginPathsChanged()
+{
+    m_applyButton->setEnabled(true);
+    m_changesOnRestart = true;
+}
+
 void
 PreferencesDialog::okClicked()
 {
@@ -1005,6 +1023,8 @@ PreferencesDialog::applyClicked()
         emit coloursChanged();
         m_coloursChanged = false;
     }
+
+    PluginPathSetter::savePathSettings(m_pluginPathConfigurator->getPaths());
 }    
 
 void
