@@ -13,16 +13,21 @@
     COPYING included with this distribution for more information.
 */
 
-#ifndef _POWER_OF_TWO_ZOOM_CONSTRAINT_H_
-#define _POWER_OF_TWO_ZOOM_CONSTRAINT_H_
+#ifndef SV_POWER_OF_TWO_ZOOM_CONSTRAINT_H
+#define SV_POWER_OF_TWO_ZOOM_CONSTRAINT_H
 
 #include "base/ZoomConstraint.h"
 
 class PowerOfTwoZoomConstraint : virtual public ZoomConstraint
 {
 public:
-    virtual int getNearestBlockSize(int requestedBlockSize,
-                                       RoundingDirection dir = RoundNearest)
+    virtual ZoomLevel getNearestZoomLevel(ZoomLevel requested,
+                                          RoundingDirection dir = RoundNearest)
+	const override;
+
+protected:
+    virtual int getNearestBlockSize(int requested,
+                                    RoundingDirection dir = RoundNearest)
         const;
 };
 
