@@ -12,6 +12,8 @@
 */
 
 #include "TestFFTModel.h"
+#include "TestZoomConstraints.h"
+#include "TestWaveformOversampler.h"
 
 #include <QtTest>
 
@@ -33,6 +35,18 @@ int main(int argc, char *argv[])
         else ++bad;
     }
 
+    {
+        TestZoomConstraints t;
+        if (QTest::qExec(&t, argc, argv) == 0) ++good;
+        else ++bad;
+    }
+
+    {
+        TestWaveformOversampler t;
+        if (QTest::qExec(&t, argc, argv) == 0) ++good;
+        else ++bad;
+    }
+
     if (bad > 0) {
         SVCERR << "\n********* " << bad << " test suite(s) failed!\n" << endl;
         return 1;
