@@ -20,7 +20,11 @@
 #include <vector>
 #include <map>
 
-class KnownPluginCandidates;
+#ifdef HAVE_PLUGIN_CHECKER_HELPER
+#include "checker/knownplugincandidates.h"
+#else
+class KnownPluginCandidates {};
+#endif
 
 class PluginScan
 {
@@ -73,6 +77,12 @@ private:
 
     void clear();
 
+#ifdef HAVE_PLUGIN_CHECKER_HELPER
+    QString formatFailureReport(QString helperTag,
+                                std::vector<PluginCandidates::FailureRec>)
+        const;
+#endif
+
     mutable QMutex m_mutex; // while scanning; definitely can't multi-thread this
     
     std::map<QString, KnownPluginCandidates *> m_kp; // tag -> KnownPlugins client
