@@ -32,6 +32,9 @@
 
 #include <vector>
 
+#include <utility>
+using namespace std::rel_ops;
+
 //#define DEBUG_COLOUR_PLOT_REPAINT 1
 
 using namespace std;
@@ -297,11 +300,19 @@ Colour3DPlotRenderer::render(const LayerGeometryProvider *v,
                     pr.x(), pr.y(), pr.width(), pr.height());
 
     if (!timeConstrained && (pr != rect)) {
-        cerr << "WARNING: failed to render entire requested rect "
+        SVCERR << "WARNING: failed to render entire requested rect "
              << "even when not time-constrained" << endl;
     }
 
     MagnitudeRange range = m_magCache.getRange(reqx0, reqx1 - reqx0);
+
+#ifdef DEBUG_COLOUR_PLOT_REPAINT
+    SVDEBUG << "render: returning rect rendered as " << pr.x() << "," << pr.y()
+            << " " << pr.width() << "x" << pr.height() << endl;
+    SVDEBUG << "render: mag range from cache in x-range " << reqx0
+            << " to " << reqx1 << " is " << range.getMin() << " -> "
+            << range.getMax() << endl;
+#endif
     
     return { pr, range };
 }
@@ -315,7 +326,7 @@ Colour3DPlotRenderer::decideRenderType(const LayerGeometryProvider *v) const
     }
 
     int binResolution = model->getResolution();
-    int zoomLevel = v->getZoomLevel();
+    ZoomLevel zoomLevel = v->getZoomLevel();
     sv_samplerate_t modelRate = model->getSampleRate();
 
     double rateRatio = v->getViewManager()->getMainModelSampleRate() / modelRate;
@@ -332,12 +343,14 @@ Colour3DPlotRenderer::decideRenderType(const LayerGeometryProvider *v) const
         // explicitly requested opaque & sufficiently zoomed-in
         
         if (model->getHeight() * 3 < v->getPaintHeight() &&
-            relativeBinResolution >= 3 * zoomLevel) {
+            zoomLevel < ZoomLevel(ZoomLevel::FramesPerPixel,
+                                  int(round(relativeBinResolution / 3)))) {
             return DirectTranslucent;
         }
     }
 
-    if (relativeBinResolution > zoomLevel) {
+    if (ZoomLevel(ZoomLevel::FramesPerPixel,
+                  int(round(relativeBinResolution))) > zoomLevel) {
         return DrawBufferBinResolution;
     } else {
         return DrawBufferPixelResolution;
@@ -348,8 +361,6 @@ ColumnOp::Column
 Colour3DPlotRenderer::getColumn(int sx, int minbin, int nbins,
                                 int peakCacheIndex) const
 {
-    Profiler profiler("Colour3DPlotRenderer::getColumn");
-    
     // order:
     // get column -> scale -> normalise -> record extents ->
     // peak pick -> distribute/interpolate -> apply display gain
@@ -358,7 +369,38 @@ Colour3DPlotRenderer::getColumn(int sx, int minbin, int nbins,
     // get column -> scale -> normalise
 
     ColumnOp::Column column;
-                
+    
+    if (m_params.showDerivative && sx > 0) {
+
+        auto prev = getColumnRaw(sx - 1, minbin, nbins, peakCacheIndex);
+        column = getColumnRaw(sx, minbin, nbins, peakCacheIndex);
+        
+        for (int i = 0; i < nbins; ++i) {
+            column[i] -= prev[i];
+        }
+
+    } else {
+        column = getColumnRaw(sx, minbin, nbins, peakCacheIndex);
+    }
+
+    if (m_params.colourScale.getScale() == ColourScaleType::Phase &&
+        m_sources.fft) {
+        return column;
+    } else {
+        column = ColumnOp::applyGain(column, m_params.scaleFactor);
+        column = ColumnOp::normalize(column, m_params.normalization);
+        return column;
+    }
+}
+
+ColumnOp::Column
+Colour3DPlotRenderer::getColumnRaw(int sx, int minbin, int nbins,
+                                   int peakCacheIndex) const
+{
+    Profiler profiler("Colour3DPlotRenderer::getColumn");
+
+    ColumnOp::Column column;
+
     if (m_params.colourScale.getScale() == ColourScaleType::Phase &&
         m_sources.fft) {
 
@@ -377,10 +419,6 @@ Colour3DPlotRenderer::getColumn(int sx, int minbin, int nbins,
                 
         column = vector<float>(fullColumn.data() + minbin,
                                fullColumn.data() + minbin + nbins);
-
-        column = ColumnOp::applyGain(column, m_params.scaleFactor);
-
-        column = ColumnOp::normalize(column, m_params.normalization);
     }
 
     return column;
@@ -464,21 +502,21 @@ Colour3DPlotRenderer::renderDirectTranslucent(const LayerGeometryProvider *v,
             psx = sx;
         }
 
-	sv_frame_t fx = sx * modelResolution + modelStart;
+        sv_frame_t fx = sx * modelResolution + modelStart;
 
-	if (fx + modelResolution <= modelStart || fx > modelEnd) continue;
+        if (fx + modelResolution <= modelStart || fx > modelEnd) continue;
 
         int rx0 = v->getXForFrame(int(double(fx) * rateRatio));
-	int rx1 = v->getXForFrame(int(double(fx + modelResolution + 1) * rateRatio));
+        int rx1 = v->getXForFrame(int(double(fx + modelResolution + 1) * rateRatio));
 
-	int rw = rx1 - rx0;
-	if (rw < 1) rw = 1;
+        int rw = rx1 - rx0;
+        if (rw < 1) rw = 1;
 
-	bool showLabel = (rw > 10 &&
-			  paint.fontMetrics().width("0.000000") < rw - 3 &&
-			  paint.fontMetrics().height() < (h / sh));
+        bool showLabel = (rw > 10 &&
+                          paint.fontMetrics().width("0.000000") < rw - 3 &&
+                          paint.fontMetrics().height() < (h / sh));
         
-	for (int sy = minbin; sy < minbin + nbins; ++sy) {
+        for (int sy = minbin; sy < minbin + nbins; ++sy) {
 
             int ry0 = m_sources.verticalBinLayer->getIYForBin(v, sy);
             int ry1 = m_sources.verticalBinLayer->getIYForBin(v, sy + 1);
@@ -501,30 +539,30 @@ Colour3DPlotRenderer::renderDirectTranslucent(const LayerGeometryProvider *v,
                 continue;
             }
 
-	    QColor pen(255, 255, 255, 80);
-	    QColor brush(colour);
+            QColor pen(255, 255, 255, 80);
+            QColor brush(colour);
 
             if (rw > 3 && r.height() > 3) {
                 brush.setAlpha(160);
             }
 
-	    paint.setPen(Qt::NoPen);
-	    paint.setBrush(brush);
+            paint.setPen(Qt::NoPen);
+            paint.setBrush(brush);
 
-	    if (illuminate) {
-		if (r.contains(illuminatePos)) {
-		    paint.setPen(v->getForeground());
-		}
-	    }
+            if (illuminate) {
+                if (r.contains(illuminatePos)) {
+                    paint.setPen(v->getForeground());
+                }
+            }
             
 #ifdef DEBUG_COLOUR_PLOT_REPAINT
 //            SVDEBUG << "rect " << r.x() << "," << r.y() << " "
 //                      << r.width() << "x" << r.height() << endl;
 #endif
 
-	    paint.drawRect(r);
+            paint.drawRect(r);
 
-	    if (showLabel) {
+            if (showLabel) {
                 double value = model->getValueAt(sx, sy);
                 snprintf(labelbuf, buflen, "%06f", value);
                 QString text(labelbuf);
@@ -535,8 +573,8 @@ Colour3DPlotRenderer::renderDirectTranslucent(const LayerGeometryProvider *v,
                      ry0 - h / sh - 1 + 2 + paint.fontMetrics().ascent(),
                      text,
                      PaintAssistant::OutlinedText);
-	    }
-	}
+            }
+        }
     }
 
     return magRange;
@@ -555,12 +593,12 @@ Colour3DPlotRenderer::getPreferredPeakCache(const LayerGeometryProvider *v,
     if (m_params.binDisplay == BinDisplay::PeakFrequencies) return;
     if (m_params.colourScale.getScale() == ColourScaleType::Phase) return;
     
-    int zoomLevel = v->getZoomLevel();
+    ZoomLevel zoomLevel = v->getZoomLevel();
     int binResolution = model->getResolution();
     
     for (int ix = 0; in_range_for(m_sources.peakCaches, ix); ++ix) {
         int bpp = m_sources.peakCaches[ix]->getColumnsPerPeak();
-        int equivZoom = binResolution * bpp;
+        ZoomLevel equivZoom(ZoomLevel::FramesPerPixel, binResolution * bpp);
         if (zoomLevel >= equivZoom) {
             // this peak cache would work, though it might not be best
             if (bpp > binsPerPeak) {
@@ -598,7 +636,7 @@ Colour3DPlotRenderer::renderToCachePixelResolution(const LayerGeometryProvider *
 
     const DenseThreeDimensionalModel *model = m_sources.source;
     if (!model || !model->isOK() || !model->isReady()) {
-	throw std::logic_error("no source model provided, or model not ready");
+        throw std::logic_error("no source model provided, or model not ready");
     }
 
     int h = v->getPaintHeight();
@@ -739,7 +777,7 @@ Colour3DPlotRenderer::renderToCacheBinResolution(const LayerGeometryProvider *v,
 
     const DenseThreeDimensionalModel *model = m_sources.source;
     if (!model || !model->isOK() || !model->isReady()) {
-	throw std::logic_error("no source model provided, or model not ready");
+        throw std::logic_error("no source model provided, or model not ready");
     }
 
     // The draw buffer will contain a fragment at bin resolution. We
@@ -759,6 +797,11 @@ Colour3DPlotRenderer::renderToCacheBinResolution(const LayerGeometryProvider *v,
     int drawBufferWidth;
     int binResolution = model->getResolution();
 
+    // These loops should eventually terminate provided that
+    // getFrameForX always returns a multiple of the zoom level,
+    // i.e. there is some x for which getFrameForX(x) == 0 and
+    // subsequent return values are equally spaced
+    
     for (int x = x0; ; --x) {
         sv_frame_t f = v->getFrameForX(x);
         if ((f / binResolution) * binResolution == f) {
@@ -769,6 +812,7 @@ Colour3DPlotRenderer::renderToCacheBinResolution(const LayerGeometryProvider *v,
             }
         }
     }
+    
     for (int x = x0 + repaintWidth; ; ++x) {
         sv_frame_t f = v->getFrameForX(x);
         if ((f / binResolution) * binResolution == f) {
@@ -779,6 +823,7 @@ Colour3DPlotRenderer::renderToCacheBinResolution(const LayerGeometryProvider *v,
             }
         }
     }
+
     drawBufferWidth = int
         ((rightBoundaryFrame - leftBoundaryFrame) / binResolution);
     
