@@ -51,7 +51,7 @@ class RangeMapper;
  */
 
 class Layer : public PropertyContainer,
-	      public XmlExportable
+              public XmlExportable
 {
     Q_OBJECT
 
@@ -61,7 +61,7 @@ public:
 
     virtual const Model *getModel() const = 0;
     Model *getModel() {
-	return const_cast<Model *>(const_cast<const Layer *>(this)->getModel());
+        return const_cast<Model *>(const_cast<const Layer *>(this)->getModel());
     }
     
     /**
@@ -106,13 +106,13 @@ public:
     virtual void setSynchronousPainting(bool /* synchronous */) { }
 
     enum VerticalPosition {
-	PositionTop, PositionMiddle, PositionBottom
+        PositionTop, PositionMiddle, PositionBottom
     };
     virtual VerticalPosition getPreferredTimeRulerPosition() const {
-	return PositionMiddle;
+        return PositionMiddle;
     }
     virtual VerticalPosition getPreferredFrameCountPosition() const {
-	return PositionBottom;
+        return PositionBottom;
     }
     virtual bool hasLightBackground() const {
         return true;
@@ -122,7 +122,7 @@ public:
 
     virtual QString getPropertyContainerName() const {
         if (m_presentationName != "") return m_presentationName;
-	else return objectName();
+        else return objectName();
     }
 
     virtual void setPresentationName(QString name);
@@ -136,6 +136,8 @@ public:
     virtual void paintVerticalScale(LayerGeometryProvider *, bool /* detailed */,
                                     QPainter &, QRect) const { }
 
+    virtual int getHorizontalScaleHeight(LayerGeometryProvider *, QPainter &) const { return 0; }
+    
     virtual bool getCrosshairExtents(LayerGeometryProvider *, QPainter &, QPoint /* cursorPos */,
                                      std::vector<QRect> &) const {
         return false;
@@ -149,7 +151,7 @@ public:
                                                QPoint now) const;
 
     virtual QString getFeatureDescription(LayerGeometryProvider *, QPoint &) const {
-	return "";
+        return "";
     }
 
     virtual QString getLabelPreceding(sv_frame_t /* frame */) const {
@@ -157,10 +159,10 @@ public:
     }
 
     enum SnapType {
-	SnapLeft,
-	SnapRight,
-	SnapNearest,
-	SnapNeighbouring
+        SnapLeft,
+        SnapRight,
+        SnapNearest,
+        SnapNeighbouring
     };
 
     /**
@@ -183,11 +185,11 @@ public:
      * the resolution of the model in this layer in sample frames.
      */
     virtual bool snapToFeatureFrame(LayerGeometryProvider * /* v */,
-				    sv_frame_t & /* frame */,
-				    int &resolution,
-				    SnapType /* snap */) const {
-	resolution = 1;
-	return false;
+                                    sv_frame_t & /* frame */,
+                                    int &resolution,
+                                    SnapType /* snap */) const {
+        resolution = 1;
+        return false;
     }
 
     /**
@@ -210,8 +212,8 @@ public:
                                       sv_frame_t & /* source frame */,
                                       int &resolution,
                                       SnapType /* snap */) const {
-	resolution = 1;
-	return false;
+        resolution = 1;
+        return false;
     }
 
     // Draw, erase, and edit modes:
@@ -412,10 +414,31 @@ public:
 
     virtual PlayParameters *getPlayParameters();
 
+    /**
+     * True if this layer will need to place text labels when it is
+     * painted. The view will take into account how many layers are
+     * requesting this, and will provide a distinct y-coord to each
+     * layer on request via View::getTextLabelHeight().
+     */
     virtual bool needsTextLabelHeight() const { return false; }
 
+    /**
+     * Return true if the X axis on the layer is time proportional to
+     * audio frames, false otherwise. Almost all layer types return
+     * true here: the exceptions are spectrum and slice layers.
+     */
     virtual bool hasTimeXAxis() const { return true; }
 
+    /**
+     * Update the X and Y axis scales, where appropriate, to focus on
+     * the given rectangular region. This should *only* be overridden
+     * by layers whose hasTimeXAxis() returns false - the pane handles
+     * zooming appropriately in every "normal" case.
+     */
+    virtual void zoomToRegion(const LayerGeometryProvider *, QRect) {
+        return;
+    }
+
     /**
      * Return the minimum and maximum values for the y axis of the
      * model in this layer, as well as whether the layer is configured
@@ -626,6 +649,8 @@ protected:
     void paintMeasurementRect(LayerGeometryProvider *v, QPainter &paint,
                               const MeasureRect &r, bool focus) const;
 
+    bool valueExtentsMatchMine(LayerGeometryProvider *v) const;
+    
     QString m_presentationName;
 
 private:
