@@ -73,7 +73,9 @@ public:
     virtual void setVerticalZoomStep(int);
     virtual RangeMapper *getNewVerticalZoomRangeMapper() const;
 
-    virtual bool hasTimeXAxis() const { return false; }
+    virtual bool hasTimeXAxis() const override { return false; }
+
+    virtual void zoomToRegion(const LayerGeometryProvider *, QRect) override;
 
     virtual bool isLayerScrollable(const LayerGeometryProvider *) const { return false; }
 
@@ -119,9 +121,22 @@ public slots:
     void modelAboutToBeDeleted(Model *);
 
 protected:
+    /// Convert a (possibly non-integral) bin into x-coord. May be overridden
     virtual double getXForBin(const LayerGeometryProvider *, double bin) const;
+    
+    /// Convert an x-coord into (possibly non-integral) bin. May be overridden
     virtual double getBinForX(const LayerGeometryProvider *, double x) const;
 
+    /// Convert a point such as a bin number into x-coord, given max &
+    /// min. For use by getXForBin etc
+    double getXForScalePoint(const LayerGeometryProvider *,
+                             double p, double pmin, double pmax) const;
+
+    /// Convert an x-coord into a point such as a bin number, given
+    /// max & min. For use by getBinForX etc
+    double getScalePointForX(const LayerGeometryProvider *,
+                             double x, double pmin, double pmax) const;
+
     virtual double getYForValue(const LayerGeometryProvider *v, double value, double &norm) const;
     virtual double getValueForY(const LayerGeometryProvider *v, double y) const;
     
@@ -140,8 +155,26 @@ protected:
 
     virtual int getDefaultColourHint(bool dark, bool &impose);
 
+    // Determine how the bins are lined up
+    // horizontally. BinsCentredOnScalePoint means we operate like a
+    // spectrum, where a bin maps to a specific frequency, and so the
+    // bin should be visually centred on the scale point that
+    // corresponds to that frequency. BinsSpanScalePoints means we
+    // have numbered or labelled bins that are not mapped to a
+    // continuous scale, like a typical chromagram output, and so bin
+    // N spans from scale point N to N+1.  This is a fundamental
+    // quality of the class or input data, not a user-configurable
+    // property.
+    //
+    enum BinAlignment {
+        BinsCentredOnScalePoints,
+        BinsSpanScalePoints
+    };
+
     const DenseThreeDimensionalModel *m_sliceableModel;
+    BinAlignment                      m_binAlignment;
     int                               m_colourMap;
+    bool                              m_colourInverted;
     EnergyScale                       m_energyScale;
     SamplingMode                      m_samplingMode;
     PlotStyle                         m_plotStyle;
