@@ -13,8 +13,8 @@
     COPYING included with this distribution for more information.
 */
 
-#ifndef _PANE_H_
-#define _PANE_H_
+#ifndef SV_PANE_H
+#define SV_PANE_H
 
 #include <QFrame>
 #include <QPoint>
@@ -87,7 +87,7 @@ public slots:
     // view slots
     virtual void toolModeChanged() override;
     virtual void zoomWheelsEnabledChanged() override;
-    virtual void viewZoomLevelChanged(View *v, int z, bool locked) override;
+    virtual void viewZoomLevelChanged(View *, ZoomLevel, bool locked) override;
     virtual void modelAlignmentCompletionChanged() override;
 
     // local slots, not overrides
@@ -212,7 +212,7 @@ protected:
 
     bool m_playbackFrameMoveScheduled;
     sv_frame_t m_playbackFrameMoveTo;
-
+    
     static QCursor *m_measureCursor1;
     static QCursor *m_measureCursor2;
 };
